# generator.py - generate output files

# Copyright (c) 2007-2008, Jason Breti (http://breti.org/codegen)
# All rights reserved.
# This code is released under the BSD License, as specified in the
# license.txt file.

import os, sys, tempfile

from parser import Parser

from utilities import changeFileIfDifferent

class Generator:
  def generateChart(self, stateChartXML, fileName, template):
    parser = Parser()

    template.stateChartXML = stateChartXML
    template.userCode      = parser.parseInputFile(fileName)

    self.generateOutputFile(fileName, template)

  def generateState(self, stateChartXML, state, fileName, template):
    parser = Parser()

    template.state         = state
    template.stateChartXML = stateChartXML
    template.userCode      = parser.parseInputFile(fileName)
    template.searchPrefix  = parser.searchPrefix
    
    self.generateOutputFile(fileName, template)

  def generateOutputFile(self, fileName, template):
      # Generate the file to a temporary file
      (tempDir, tempBaseFN) = os.path.split(fileName)
      
      (fd, tempFileName) = tempfile.mkstemp(prefix=tempBaseFN+'.tempgen.',
                                            dir=tempDir,
                                            text=True)
      
      outputFile = open(tempFileName,'w')
      try:
        outputFile.write(str(template))
      finally:
        outputFile.close()

      # Replace the original generated file with the new temporary file
      # if the original file has changed
      if changeFileIfDifferent(fileName, tempFileName):
        print 'Generated:', fileName
