# utilities.py - File Utilities

# Copyright (c) 2008-2009, Jason Breti (http://breti.org/codegen)
# This code is released under the BSD License, as specified in the
# license.txt file.

import os, filecmp

import xml.etree.ElementTree as et

def changeFileIfDifferent(actualFileName,
                          tempFileName,
                          useBackup=True,
                          useDebug=False):
  
  fileChanged = False

  # if actualFilename already exists then compare it with the previous
  # actualFilename and overwrite it if they are different

  if (os.path.exists(actualFileName) and os.access(actualFileName,os.W_OK)):
    if not filecmp.cmp(tempFileName, actualFileName, shallow=False):
      if useDebug:
        print 'Generating', actualFileName
      if useBackup:
        backupFileName = _getBackupFileName(actualFileName)

        if useDebug:
          print 'Creating backup file', backupFileName, 'for', actualFileName

        os.rename(actualFileName, backupFileName)

      os.rename(tempFileName, actualFileName)
      fileChanged = True
    else:
      if useDebug:
        print 'No changes required for', actualFileName
      os.remove(tempFileName)

  # if actualFilename doesn't exist then rename tempFilename
  # to the actualFilename

  elif os.path.exists(actualFileName) == False:
    if useDebug:
      print 'New file', actualFileName, 'is generated'
    os.rename(tempFileName, actualFileName)
    fileChanged = True

  else:
    print "Unable to write to:", actualFileName
    os.remove(tempFileName)
    
  return fileChanged

#------------------------------------------------------------------------------
def _getBackupFileName(fileName):
  startVersion = 1
  backupFileName = fileName + '.backup.' + '%03d'%startVersion
  while os.path.isfile(backupFileName):
    startVersion += 1
    backupFileName = fileName + '.backup.' + '%03d'%startVersion

  return backupFileName

